<?php

declare(strict_types=1);

namespace Rector\Symfony\CodeQuality\Rector\Class_;

use PhpParser\Node;
use PhpParser\Node\Stmt\Class_;
use Rector\Rector\AbstractRector;
use Symplify\RuleDocGenerator\ValueObject\CodeSample\CodeSample;
use Symplify\RuleDocGenerator\ValueObject\RuleDefinition;

/**
 * @deprecated This rule is deprecated since Rector 1.1.2, as Symfony 6.1 introduced native attribute,
 *  more reliable and validated
 */
final class MakeCommandLazyRector extends AbstractRector
{
    private bool $hasWarned = false;

    public function getRuleDefinition(): RuleDefinition
    {
        return new RuleDefinition('Make Symfony commands lazy', [
            new CodeSample(
                <<<'CODE_SAMPLE'
use Symfony\Component\Console\Command\Command

final class SunshineCommand extends Command
{
    public function configure()
    {
        $this->setName('sunshine');
    }
}
CODE_SAMPLE
                ,
                <<<'CODE_SAMPLE'
use Symfony\Component\Console\Command\Command

final class SunshineCommand extends Command
{
    protected static $defaultName = 'sunshine';
    public function configure()
    {
    }
}
CODE_SAMPLE
            ),
        ]);
    }

    /**
     * @return array<class-string<Node>>
     */
    public function getNodeTypes(): array
    {
        return [Class_::class];
    }

    /**
     * @param Class_ $node
     */
    public function refactor(Node $node): ?Node
    {
        if ($this->hasWarned) {
            return null;
        }

        trigger_error(
            sprintf(
                'The "%s" rule was deprecated, as its only dead middle step before more solid PHP attributes.',
                self::class,
            )
        );

        sleep(3);

        $this->hasWarned = true;

        return null;
    }
}
