<?php

declare(strict_types=1);

namespace Rector\Symfony\Symfony40\Rector\MethodCall;

use PhpParser\Node;
use PhpParser\Node\Expr\MethodCall;
use Rector\Rector\AbstractRector;
use Symplify\RuleDocGenerator\ValueObject\CodeSample\CodeSample;
use Symplify\RuleDocGenerator\ValueObject\RuleDefinition;

/**
 * @deprecated This rule is deprecated since Rector 1.1.2, as it can create invalid code.
 * This needs a manual change
 *
 * @see https://github.com/pact-foundation/pact-php/pull/61/files.
 */
final class ProcessBuilderGetProcessRector extends AbstractRector
{
    private bool $hasWarned = false;

    public function getRuleDefinition(): RuleDefinition
    {
        return new RuleDefinition(
            'Removes `$processBuilder->getProcess()` calls to $processBuilder in Process in Symfony, because ProcessBuilder was removed. This is part of multi-step Rector and has very narrow focus.',
            [
                new CodeSample(
                    <<<'CODE_SAMPLE'
$processBuilder = new Symfony\Component\Process\ProcessBuilder;
$process = $processBuilder->getProcess();
$commamdLine = $processBuilder->getProcess()->getCommandLine();
CODE_SAMPLE
                    ,
                    <<<'CODE_SAMPLE'
$processBuilder = new Symfony\Component\Process\ProcessBuilder;
$process = $processBuilder;
$commamdLine = $processBuilder->getCommandLine();
CODE_SAMPLE
                ),
            ]
        );
    }

    /**
     * @return array<class-string<Node>>
     */
    public function getNodeTypes(): array
    {
        return [MethodCall::class];
    }

    /**
     * @param MethodCall $node
     */
    public function refactor(Node $node): ?Node
    {
        if ($this->hasWarned) {
            return null;
        }

        trigger_error(
            sprintf(
                'The "%s" rule was deprecated, as it cannot change fluent code builder in a reliable way.',
                self::class,
            )
        );

        sleep(3);

        $this->hasWarned = true;

        return null;

    }
}
